/*******************************************************************************

  Copyright(c) 1999 - 2015 Intel Corporation. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of the GNU General Public License as published by the Free 
  Software Foundation; either version 2 of the License, or (at your option) 
  any later version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in the
  file called LICENSE.
  
  Contact Information:
  e1000 mailing list <e1000-devel@lists.sourceforge.net>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

*******************************************************************************/

#include <stdio.h>
#include "ethregs.h"


#define REGVAL(base, offset) *((uint32_t *) (base + offset))
#define printval32(i, name, val) \
    printf("\t(0x%05x) %-15s%08x\n", i, name, val)

extern int clr;
extern int offset;

static void printit32_array (int, char *, volatile void *, int);
static void printit32_array (int i, char *name, volatile void *mem, int array_len)
{
	int j,n=0, alreadyzero=0;
	char regname[16];

	printf ("name: %s  len: %d\n", name, array_len);

	for (j=i, n=0; n < array_len; n++, j += sizeof(u32)) {
		if (*(u32 *)(mem + j) == 0) {
			switch (alreadyzero) {
			case 0:
				alreadyzero = 1;
				break;
			case 1:
				printf("\t...\n");
				alreadyzero = 2;
				continue;
			case 2:
				continue;
			}
		} else
			alreadyzero = 0;
		sprintf(regname, "%s[%d]", name, n);
		printit32(j, regname, mem);
	}
}

void dump_82576vf_registers(volatile void *mem, u32 size)
{
	unsigned int i, n;
	uint32_t val;
	char name[16];

	for (i = 0; i < size; i += sizeof(u32)) {
		switch (i & 0xffffffff) {
		case E1000_CTRL:
			sprintf(name, "CTRL"); break;
		case E1000_STATUS:
			sprintf(name, "STATUS"); break;
		case E1000_FRTIMER:
			sprintf(name, "FRTIMER"); break;
		case E1000_EICR:
			if (!clr) continue;
			sprintf(name, "EICR"); break;
		case E1000_EICS:
			sprintf(name, "EICS"); break;
		case E1000_EIMS:
			sprintf(name, "EIMS"); break;
		case E1000_EIMC:
			sprintf(name, "EIMC"); break;
		case E1000_EIAC:
			sprintf(name, "EIAC"); break;
		case E1000_EIAM:
			sprintf(name, "EIAM"); break;

		case E1000_IVAR0:
			sprintf(name, "IVAR"); break;
		case E1000_IVAR_MISC:
			sprintf(name, "IVAR_MISC"); break;
		case E1000_PBACL:
			sprintf(name, "PBACL"); break;

		case E1000_PSRTYPE(0):
			sprintf(name, "PSRTYP0"); break;

		case E1000_EITR(0):
			sprintf(name, "EITR");
			printit32_array(i, name, mem, 2);
			continue;
		case E1000_RDBAL(0):
			sprintf(name, "RDBAL[%d]", 0);
			printit32(E1000_RDBAL(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_RDBAL(n), mem))) {
					sprintf(name, "RDBAL[%d]", n);
					printval32(E1000_RDBAL(n), name, val);
				}
			}
			continue;
		case E1000_RDBAH(0):
			sprintf(name, "RDBAH[%d]", 0);
			printit32(E1000_RDBAH(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_RDBAH(n), mem))) {
					sprintf(name, "RDBAH[%d]", n);
					printval32(E1000_RDBAH(n), name, val);
				}
			}
			continue;
		case E1000_RDLEN(0):
			sprintf(name, "RDLEN[%d]", 0);
			printit32(E1000_RDLEN(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_RDLEN(n), mem))) {
					sprintf(name, "RDLEN[%d]", n);
					printval32(E1000_RDLEN(n), name, val);
				}
			}
			continue;
		case E1000_RDH(0):
			sprintf(name, "RDH[%d]", 0);
			printit32(E1000_RDH(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_RDH(n), mem))) {
					sprintf(name, "RDH[%d]", n);
					printval32(E1000_RDH(n), name, val);
				}
			}
			continue;
		case E1000_RDT(0):
			sprintf(name, "RDT[%d]", 0);
			printit32(E1000_RDT(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_RDT(n), mem))) {
					sprintf(name, "RDT[%d]", n);
					printval32(E1000_RDT(n), name, val);
				}
			}
			continue;
		case E1000_SRRCTL(0):
			sprintf(name, "SRRCTL[%d]", 0);
			printit32(E1000_SRRCTL(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_SRRCTL(n), mem))) {
					sprintf(name, "SRRCTL[%d]", n);
					printval32(E1000_SRRCTL(n), name, val);
				}
			}
			continue;
		case E1000_RXDCTL(0):
			sprintf(name, "RXDCTL[%d]", 0);
			printit32(E1000_RXDCTL(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_RXDCTL(n), mem))) {
					sprintf(name, "RXDCTL[%d]", n);
					printval32(E1000_RXDCTL(n), name, val);
				}
			}
			continue;
		case E1000_DCA_RXCTRL(0):
			sprintf(name, "DCA_RXCTL[%d]", 0);
			printit32(E1000_DCA_RXCTRL(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_DCA_RXCTRL(n), mem))) {
					sprintf(name, "DCA_RXCTL[%d]", n);
					printval32(E1000_DCA_RXCTRL(n), name, val);
				}
			}
			continue;
		case E1000_RQDPC(0):
			sprintf(name, "RQDPC[%d]", 0);
			printit32(E1000_RQDPC(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_RQDPC(n), mem))) {
					sprintf(name, "RQDPC[%d]", n);
					printval32(E1000_RQDPC(n), name, val);
				}
			}
			continue;
		case E1000_TDBAL(0):
			sprintf(name, "TDBAL[%d]", 0);
			printit32(E1000_TDBAL(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_TDBAL(n), mem))) {
					sprintf(name, "TDBAL[%d]", n);
					printval32(E1000_TDBAL(n), name, val);
				}
			}
			continue;
		case E1000_TDBAH(0):
			sprintf(name, "TDBAH[%d]", 0);
			printit32(E1000_TDBAH(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_TDBAH(n), mem))) {
					sprintf(name, "TDBAH[%d]", n);
					printval32(E1000_TDBAH(n), name, val);
				}
			}
			continue;
		case E1000_TDLEN(0):
			sprintf(name, "TDLEN[%d]", 0);
			printit32(E1000_TDLEN(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_TDLEN(n), mem))) {
					sprintf(name, "TDLEN[%d]", n);
					printval32(E1000_TDLEN(n), name, val);
				}
			}
			continue;
		case E1000_TDH(0):
			sprintf(name, "TDH[%d]", 0);
			printit32(E1000_TDH(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_TDH(n), mem))) {
					sprintf(name, "TDH[%d]", n);
					printval32(E1000_TDH(n), name, val);
				}
			}
			continue;
		case E1000_DCA_TXCTRL(0):
			sprintf(name, "DCA_TXCTRL[%d]", 0);
			printit32(E1000_DCA_TXCTRL(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_DCA_TXCTRL(n), mem))) {
					sprintf(name, "DCA_TXCTRL[%d]", n);
					printval32(E1000_DCA_TXCTRL(n), name, val);
				}
			}
			continue;
		case E1000_TDT(0):
			sprintf(name, "TDT[%d]", 0);
			printit32(E1000_TDT(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_TDT(n), mem))) {
					sprintf(name, "TDT[%d]", n);
					printval32(E1000_TDT(n), name, val);
				}
			}
			continue;
		case E1000_TXDCTL(0):
			sprintf(name, "TXDCTL[%d]", 0);
			printit32(E1000_TXDCTL(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_TXDCTL(n), mem))) {
					sprintf(name, "TXDCTL[%d]", n);
					printval32(E1000_TXDCTL(n), name, val);
				}
			}
			continue;
		case E1000_TDWBAL(0):
			sprintf(name, "TDWBAL[%d]", 0);
			printit32(E1000_TDWBAL(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_TDWBAL(n), mem))) {
					sprintf(name, "TDWBAL[%d]", n);
					printval32(E1000_TDWBAL(n), name, val);
				}
			}
			continue;
		case E1000_TDWBAH(0):
			sprintf(name, "TDWBAH[%d]", 0);
			printit32(E1000_TDWBAH(0), name, mem);
			for (n=1; n<2; n++) {
				if (0 != (val = REGVAL(E1000_TDWBAH(n), mem))) {
					sprintf(name, "TDWBAH[%d]", n);
					printval32(E1000_TDWBAH(n), name, val);
				}
			}
			continue;

		/* subset of virtualization registers: just the ones #defined
		 * in shared code */

		case E1000_V2PMAILBOX(0):
			sprintf(name, "V2PMAILBOX0"); break;
		case E1000_VMBMEM(0):
			sprintf(name, "VMBMEM0"); break;

		/* statistics */

		case E1000_VFGPRC:
			if (!clr) continue;
			sprintf(name, "VFGPRC"); break;
		case E1000_VFMPRC:
			if (!clr) continue;
			sprintf(name, "VFMPRC"); break;
		case E1000_VFGPTC:
			if (!clr) continue;
			sprintf(name, "VFGPTC"); break;
		case E1000_VFGORC:
			if (!clr) continue;
			sprintf(name, "VFGORC"); break;
		case E1000_VFGOTC:
			if (!clr) continue;
			sprintf(name, "VFGOTC"); break;
		case E1000_VFGOTLBC:
			if (!clr) continue;
			sprintf(name, "VFGOTLBC"); break;
		case E1000_VFGPTLBC:
			if (!clr) continue;
			sprintf(name, "VFGPTLBC"); break;
		case E1000_VFGORLBC:
			if (!clr) continue;
			sprintf(name, "VFGORLBC"); break;
		case E1000_VFGPRLBC:
			if (!clr) continue;
			sprintf(name, "VFGPRLBC"); break;

		default:
			continue;
		}
		printit32(i, name, mem);
	}
}

// vi:ts=4:sw=4:ht=4:
